from ogn.parser.pattern import PATTERN_FLARM_POSITION_COMMENT
from ogn.parser.utils import FPM_TO_MS, HPM_TO_DEGS

from .base import BaseParser


class FlarmParser(BaseParser):
    def __init__(self):
        self.beacon_type = 'flarm'
        self.position_pattern = PATTERN_FLARM_POSITION_COMMENT

    def parse_position(self, aprs_comment):
        match = self.position_pattern.match(aprs_comment)

        result = {}
        if match.group('details'):
            result.update({
                'address_type': int(match.group('details'), 16) & 0b00000011,
                'aircraft_type': (int(match.group('details'), 16) & 0b00111100) >> 2,
                'no-tracking': (int(match.group('details'), 16) & 0b01000000) >> 6 == 1,
                'stealth': (int(match.group('details'), 16) & 0b10000000) >> 7 == 1,
                'address': match.group('address'),
            })
        if match.group('climb_rate'): result['climb_rate'] = int(match.group('climb_rate')) * FPM_TO_MS
        if match.group('turn_rate'): result['turn_rate'] = float(match.group('turn_rate')) * HPM_TO_DEGS
        if match.group('signal_quality'): result['signal_quality'] = float(match.group('signal_quality'))
        if match.group('error_count'): result['error_count'] = int(match.group('error_count'))
        if match.group('frequency_offset'): result['frequency_offset'] = float(match.group('frequency_offset'))
        if match.group('gps_quality'):
            result.update({
                'gps_quality': {
                    'horizontal': int(match.group('gps_quality_horizontal')),
                    'vertical': int(match.group('gps_quality_vertical'))
                }
            })
        if match.group('software_version'): result['software_version'] = float(match.group('software_version'))
        if match.group('hardware_version'): result['hardware_version'] = int(match.group('hardware_version'), 16)
        if match.group('real_address'): result['real_address'] = match.group('real_address')
        if match.group('signal_power'): result['signal_power'] = float(match.group('signal_power'))
        return result



import unittest

from ogn.parser.aprs_comment.receiver_parser import ReceiverParser


class TestStringMethods(unittest.TestCase):
    def test_position_comment(self):
        message = ReceiverParser().parse_position("Antenna: chinese, on a pylon, 20 meter above ground")

        self.assertEqual(message['user_comment'], "Antenna: chinese, on a pylon, 20 meter above ground")

    def test_position_comment_empty(self):
        message = ReceiverParser().parse_position("")

        self.assertIsNotNone(message)

    def test_status_comment(self):
        message = ReceiverParser().parse_status("v0.2.7.RPI-GPU CPU:0.7 RAM:770.2/968.2MB NTP:1.8ms/-3.3ppm +55.7C 7/8Acfts[1h] RF:+54-1.1ppm/-0.16dB/+7.1dB@10km[19481]/+16.8dB@10km[7/13]")

        self.assertEqual(message['version'], "0.2.7")
        self.assertEqual(message['platform'], 'RPI-GPU')
        self.assertEqual(message['cpu_load'], 0.7)
        self.assertEqual(message['free_ram'], 770.2)
        self.assertEqual(message['total_ram'], 968.2)
        self.assertEqual(message['ntp_error'], 1.8)
        self.assertEqual(message['rt_crystal_correction'], -3.3)
        self.assertEqual(message['cpu_temp'], 55.7)
        self.assertEqual(message['senders_visible'], 7)
        self.assertEqual(message['senders_total'], 8)
        self.assertEqual(message['rec_crystal_correction'], 54)
        self.assertEqual(message['rec_crystal_correction_fine'], -1.1)
        self.assertEqual(message['rec_input_noise'], -0.16)
        self.assertEqual(message['senders_signal'], 7.1)
        self.assertEqual(message['senders_messages'], 19481)
        self.assertEqual(message['good_senders_signal'], 16.8)
        self.assertEqual(message['good_senders'], 7)
        self.assertEqual(message['good_and_bad_senders'], 13)


if __name__ == '__main__':
    unittest.main()



import requests

DDB_URL = "http://ddb.glidernet.org/download/?j=1"


def get_ddb_devices():
    r = requests.get(DDB_URL)
    for device in r.json()['devices']:
        device.update({'identified': device['identified'] == 'Y',
                       'tracked': device['tracked'] == 'Y'})
        yield device



from ogn.parser.utils import FPM_TO_MS, HPM_TO_DEGS
from ogn.parser.pattern import PATTERN_RECEIVER_BEACON, PATTERN_AIRCRAFT_BEACON

from .base import BaseParser


class OgnParser(BaseParser):
    def __init__(self):
        self.beacon_type = None
        self.aircraft_pattern = PATTERN_AIRCRAFT_BEACON
        self.receiver_pattern = PATTERN_RECEIVER_BEACON

    def parse(self, aprs_comment, aprs_type):
        if not aprs_comment:
            return {'beacon_type': 'aprs_receiver'}

        ab_data = self.parse_aircraft_beacon(aprs_comment)
        if ab_data:
            ab_data.update({'beacon_type': 'aprs_aircraft'})
            return ab_data

        rb_data = self.parse_receiver_beacon(aprs_comment)
        if rb_data:
            rb_data.update({'beacon_type': 'aprs_receiver'})
            return rb_data
        else:
            return {'user_comment': aprs_comment,
                    'beacon_type': 'aprs_receiver'}

    def parse_aircraft_beacon(self, aprs_comment):
        match = self.aircraft_pattern.match(aprs_comment)
        if match:
            result = {}
            if match.group('details'):
                result.update({
                    'address_type': int(match.group('details'), 16) & 0b00000011,
                    'aircraft_type': (int(match.group('details'), 16) & 0b00111100) >> 2,
                    'no-tracking': (int(match.group('details'), 16) & 0b01000000) >> 6 == 1,
                    'stealth': (int(match.group('details'), 16) & 0b10000000) >> 7 == 1,
                    'address': match.group('address'),
                })
            if match.group('climb_rate'): result['climb_rate'] = int(match.group('climb_rate')) * FPM_TO_MS
            if match.group('turn_rate'): result['turn_rate'] = float(match.group('turn_rate')) * HPM_TO_DEGS
            if match.group('flight_level'): result['flightlevel'] = float(match.group('flight_level'))
            if match.group('signal_quality'): result['signal_quality'] = float(match.group('signal_quality'))
            if match.group('errors'): result['error_count'] = int(match.group('errors'))
            if match.group('frequency_offset'): result['frequency_offset'] = float(match.group('frequency_offset'))
            if match.group('gps_quality'):
                result.update({
                    'gps_quality': {
                        'horizontal': int(match.group('gps_quality_horizontal')),
                        'vertical': int(match.group('gps_quality_vertical'))
                    }
                })
            if match.group('flarm_software_version'): result['software_version'] = float(match.group('flarm_software_version'))
            if match.group('flarm_hardware_version'): result['hardware_version'] = int(match.group('flarm_hardware_version'), 16)
            if match.group('flarm_id'): result['real_address'] = match.group('flarm_id')
            if match.group('signal_power'): result['signal_power'] = float(match.group('signal_power'))
            if match.group('proximity'): result['proximity'] = [hear[4:] for hear in match.group('proximity').split(' ')]
            return result
        else:
            return None

    def parse_receiver_beacon(self, aprs_comment):
        match = self.receiver_pattern.match(aprs_comment)
        if match:
            result = {
                'version': match.group('version'),
                'platform': match.group('platform'),
                'cpu_load': float(match.group('cpu_load')),
                'free_ram': float(match.group('ram_free')),
                'total_ram': float(match.group('ram_total')),
                'ntp_error': float(match.group('ntp_offset')),
                'rt_crystal_correction': float(match.group('ntp_correction'))
            }
            if match.group('voltage'): result['voltage'] = float(match.group('voltage'))
            if match.group('amperage'): result['amperage'] = float(match.group('amperage'))
            if match.group('cpu_temperature'): result['cpu_temp'] = float(match.group('cpu_temperature'))
            if match.group('visible_senders'): result['senders_visible'] = int(match.group('visible_senders'))
            if match.group('senders'): result['senders_total'] = int(match.group('senders'))
            if match.group('latency'): result['latency'] = float(match.group('latency'))
            if match.group('rf_correction_manual'): result['rec_crystal_correction'] = int(match.group('rf_correction_manual'))
            if match.group('rf_correction_automatic'): result['rec_crystal_correction_fine'] = float(match.group('rf_correction_automatic'))
            if match.group('signal_quality'): result['rec_input_noise'] = float(match.group('signal_quality'))
            if match.group('senders_signal_quality'): result['senders_signal'] = float(match.group('senders_signal_quality'))
            if match.group('senders_messages'): result['senders_messages'] = float(match.group('senders_messages'))
            if match.group('good_senders_signal_quality'): result['good_senders_signal'] = float(match.group('good_senders_signal_quality'))
            if match.group('good_senders'): result['good_senders'] = float(match.group('good_senders'))
            if match.group('good_and_bad_senders'): result['good_and_bad_senders'] = float(match.group('good_and_bad_senders'))
            return result
        else:
            return None



APRS_SERVER_HOST = 'aprs.glidernet.org'
APRS_SERVER_PORT_FULL_FEED = 10152
APRS_SERVER_PORT_CLIENT_DEFINED_FILTERS = 14580

APRS_APP_NAME = 'python-ogn-client'

PACKAGE_VERSION = '1.1.0'
APRS_APP_VER = PACKAGE_VERSION[:3]

APRS_KEEPALIVE_TIME = 240

TELNET_SERVER_HOST = 'localhost'
TELNET_SERVER_PORT = 50001



from datetime import datetime

from ogn.parser.utils import createTimestamp
from ogn.parser.pattern import PATTERN_TELNET_50001

telnet_50001_pattern = PATTERN_TELNET_50001


def parse(telnet_data):
    reference_timestamp = datetime.utcnow()

    match = telnet_50001_pattern.match(telnet_data)
    if match:
        return {'pps_offset': float(match.group('pps_offset')),
                'frequency': float(match.group('frequency')),
                'aircraft_type': int(match.group('aircraft_type')),
                'address_type': int(match.group('address_type')),
                'address': match.group('address'),
                'timestamp': createTimestamp(match.group('timestamp') + 'h', reference_timestamp),
                'latitude': float(match.group('latitude')),
                'longitude': float(match.group('longitude')),
                'altitude': int(match.group('altitude')),
                'climb_rate': float(match.group('climb_rate')),
                'ground_speed': float(match.group('ground_speed')),
                'track': float(match.group('track')),
                'turn_rate': float(match.group('turn_rate')),
                'magic_number': int(match.group('magic_number')),
                'gps_status': match.group('gps_status'),
                'channel': int(match.group('channel')),
                'flarm_timeslot': match.group('flarm_timeslot') == 'f',
                'ogn_timeslot': match.group('ogn_timeslot') == 'o',
                'frequency_offset': float(match.group('frequency_offset')),
                'decode_quality': float(match.group('decode_quality')),
                'signal_quality': float(match.group('signal_quality')),
                'demodulator_type': int(match.group('demodulator_type')),
                'error_count': float(match.group('error_count')),
                'distance': float(match.group('distance')),
                'bearing': float(match.group('bearing')),
                'phi': float(match.group('phi')),
                'multichannel': match.group('multichannel') == '+'}
    else:
        return None



import unittest
from ogn.parser.aprs_comment.inreach_parser import InreachParser


class TestStringMethods(unittest.TestCase):
    def test_position_comment(self):
        message = InreachParser().parse_position("id300434060496190 inReac True")
        self.assertEqual(message['address'], "300434060496190")
        self.assertEqual(message['model'], 'inReac')
        self.assertEqual(message['status'], True)
        self.assertEqual(message['pilot_name'], None)

        message = InreachParser().parse_position("id300434060496190 inReac True Jim Bob")
        self.assertEqual(message['address'], "300434060496190")
        self.assertEqual(message['model'], 'inReac')
        self.assertEqual(message['status'], True)
        self.assertEqual(message['pilot_name'], "Jim Bob")


if __name__ == '__main__':
    unittest.main()



from ogn.client.client import AprsClient    # noqa: F401
from ogn.client.client import TelnetClient  # noqa: F401


class CustomSettings(object):
    def __init__(self, **kw):
        self.kw = kw

    def __getattr__(self, name):
        return self.kw[name]



from ogn.parser.pattern import PATTERN_TRACKER_POSITION_COMMENT, PATTERN_TRACKER_STATUS_COMMENT
from ogn.parser.utils import FPM_TO_MS, HPM_TO_DEGS

from .base import BaseParser


class TrackerParser(BaseParser):
    def __init__(self):
        self.beacon_type = 'tracker'
        self.position_pattern = PATTERN_TRACKER_POSITION_COMMENT
        self.status_pattern = PATTERN_TRACKER_STATUS_COMMENT

    def parse_position(self, aprs_comment):
        match = self.position_pattern.match(aprs_comment)

        result = {}
        if match.group('details'):
            result.update({
                'address_type': int(match.group('details'), 16) & 0b00000011,
                'aircraft_type': (int(match.group('details'), 16) & 0b01111100) >> 2,
                'stealth': (int(match.group('details'), 16) & 0b10000000) >> 7 == 1,
                'address': match.group('address'),
            })
        if match.group('climb_rate'): result['climb_rate'] = int(match.group('climb_rate')) * FPM_TO_MS
        if match.group('turn_rate'): result['turn_rate'] = float(match.group('turn_rate')) * HPM_TO_DEGS
        if match.group('flight_level'): result['flightlevel'] = float(match.group('flight_level'))
        if match.group('signal_quality'): result['signal_quality'] = float(match.group('signal_quality'))
        if match.group('error_count'): result['error_count'] = int(match.group('error_count'))
        if match.group('frequency_offset'): result['frequency_offset'] = float(match.group('frequency_offset'))
        if match.group('gps_quality'):
            result.update({
                'gps_quality': {
                    'horizontal': int(match.group('gps_quality_horizontal')),
                    'vertical': int(match.group('gps_quality_vertical'))
                }
            })
        if match.group('signal_power'): result['signal_power'] = float(match.group('signal_power'))
        return result

    def parse_status(self, aprs_comment):
        match = self.status_pattern.match(aprs_comment)
        if match:
            result = {}

            if match.group('hardware_version'): result['hardware_version'] = int(match.group('hardware_version'))
            if match.group('software_version'): result['software_version'] = int(match.group('software_version'))
            if match.group('gps_satellites'): result['gps_satellites'] = int(match.group('gps_satellites'))
            if match.group('gps_quality'): result['gps_quality'] = int(match.group('gps_quality'))
            if match.group('gps_altitude'): result['gps_altitude'] = int(match.group('gps_altitude'))
            if match.group('pressure'): result['pressure'] = float(match.group('pressure'))
            if match.group('temperature'): result['temperature'] = float(match.group('temperature'))
            if match.group('humidity'): result['humidity'] = int(match.group('humidity'))
            if match.group('voltage'): result['voltage'] = float(match.group('voltage'))
            if match.group('transmitter_power'): result['transmitter_power'] = int(match.group('transmitter_power'))
            if match.group('noise_level'): result['noise_level'] = float(match.group('noise_level'))
            if match.group('relays'): result['relays'] = int(match.group('relays'))
            return result
        else:
            return {'comment': aprs_comment}



from ogn.parser.utils import FPM_TO_MS
from ogn.parser.pattern import PATTERN_SKYLINES_POSITION_COMMENT

from .base import BaseParser


class SkylinesParser(BaseParser):
    def __init__(self):
        self.beacon_type = 'skylines'
        self.position_pattern = PATTERN_SKYLINES_POSITION_COMMENT

    def parse_position(self, aprs_comment):
        match = self.position_pattern.match(aprs_comment)
        return {'skylines_id': match.group('skylines_id'),
                'climb_rate': int(match.group('climb_rate')) * FPM_TO_MS if match.group('climb_rate') else None}



from ogn.parser.pattern import PATTERN_SPOT_POSITION_COMMENT

from .base import BaseParser


class SpotParser(BaseParser):
    def __init__(self):
        self.beacon_type = 'spot'
        self.position_pattern = PATTERN_SPOT_POSITION_COMMENT

    def parse_position(self, aprs_comment):
        match = self.position_pattern.match(aprs_comment)
        return {'spot_id': match.group('spot_id'),
                'model': match.group('model') if match.group('model') else None,
                'status': match.group('status') if match.group('status') else None}



import re

PATTERN_APRS = re.compile(r"^(?P<callsign>.+?)>(?P<dstcall>[A-Z0-9]+),((?P<relay>[A-Za-z0-9]+)\*)?.*,(?P<receiver>.+?):(?P<aprs_type>(.))(?P<aprs_body>.*)$")
PATTERN_APRS_POSITION = re.compile(r"^(?P<time>(([0-1]\d|2[0-3])[0-5]\d[0-5]\dh|([0-2]\d|3[0-1])([0-1]\d|2[0-3])[0-5]\dz))(?P<latitude>9000\.00|[0-8]\d{3}\.\d{2})(?P<latitude_sign>N|S)(?P<symbol_table>.)(?P<longitude>18000\.00|1[0-7]\d{3}\.\d{2}|0\d{4}\.\d{2})(?P<longitude_sign>E|W)(?P<symbol>.)(?P<course_extension>(?P<course>\d{3})/(?P<ground_speed>\d{3}))?(/A=(?P<altitude>(-\d{5}|\d{6})))?(?P<pos_extension>\s!W((?P<latitude_enhancement>\d)(?P<longitude_enhancement>\d))!)?(?:\s(?P<comment>.*))?$")
PATTERN_APRS_POSITION_WEATHER = re.compile(r"^(?P<time>(([0-1]\d|2[0-3])[0-5]\d[0-5]\dh|([0-2]\d|3[0-1])([0-1]\d|2[0-3])[0-5]\dz))(?P<latitude>9000\.00|[0-8]\d{3}\.\d{2})(?P<latitude_sign>N|S)(?P<symbol_table>.)(?P<longitude>18000\.00|1[0-7]\d{3}\.\d{2}|0\d{4}\.\d{2})(?P<longitude_sign>E|W)(?P<symbol>.)(?P<wind_direction>(\d{3}|\.{3}))/(?P<wind_speed>(\d{3}|\.{3}))g(?P<wind_speed_peak>(\d{3}|\.{3}))t(?P<temperature>(\d{3}|\.{3}))(h(?P<humidity>\d{2}))?(b(?P<barometric_pressure>\d{5}))?(?:\s(?P<comment>.*))?$")
PATTERN_APRS_STATUS = re.compile(r"^(?P<time>(([0-1]\d|2[0-3])[0-5]\d[0-5]\dh|([0-2]\d|3[0-1])([0-1]\d|2[0-3])[0-5]\dz))\s(?P<comment>.*)$")

PATTERN_SERVER = re.compile(r"^# aprsc (?P<version>[a-z0-9\.\-]+) (?P<timestamp>\d+ [A-Za-z]+ \d+ \d{2}:\d{2}:\d{2} GMT) (?P<server>[A-Z0-9]+) (?P<ip_address>\d+\.\d+\.\d+\.\d+):(?P<port>\d+)$")

PATTERN_FANET_POSITION_COMMENT = re.compile(r"""
    (id(?P<details>[\dA-F]{2})(?P<address>[\dA-F]{6}?)\s?)?
    (?:(?P<climb_rate>[+-]\d+)fpm)?
""", re.VERBOSE | re.MULTILINE)

PATTERN_FANET_STATUS_COMMENT = re.compile(r"""
    (?:(Name=\"(?P<fanet_name>[^\"]*)\")\s?)?
    (?:(?P<signal_quality>[\d.]+?)dB\s?)?
    (?:(?P<frequency_offset>[+-][\d.]+?)kHz\s?)?
    (?:(?P<error_count>\d+)e\s?)?
""", re.VERBOSE | re.MULTILINE)

PATTERN_FLARM_POSITION_COMMENT = re.compile(r"""
    id(?P<details>[\dA-F]{2})(?P<address>[\dA-F]{6}?)\s?
    (?:(?P<climb_rate>[+-]\d+?)fpm\s)?
    (?:(?P<turn_rate>[+-][\d.]+?)rot\s)?
    (?:(?P<signal_quality>[\d.]+?)dB\s)?
    (?:(?P<error_count>\d+)e\s)?
    (?:(?P<frequency_offset>[+-][\d.]+?)kHz\s?)?
    (?:gps(?P<gps_quality>(?P<gps_quality_horizontal>(\d+))x(?P<gps_quality_vertical>(\d+)))\s?)?
    (?:s(?P<software_version>[\d.]+)\s?)?
    (?:h(?P<hardware_version>[\dA-F]{2})\s?)?
    (?:r(?P<real_address>[\dA-F]+)\s?)?
    (?:(?P<signal_power>[+-][\d.]+)dBm\s?)?
""", re.VERBOSE | re.MULTILINE)

PATTERN_LT24_POSITION_COMMENT = re.compile(r"""
    id(?P<lt24_id>\d+)\s
    (?P<climb_rate>[+-]\d+)fpm\s
    (?P<source>.+)
""", re.VERBOSE | re.MULTILINE)

PATTERN_NAVITER_POSITION_COMMENT = re.compile(r"""
    id(?P<details>[\dA-F]{4})(?P<address>[\dA-F]{6})\s
    (?P<climb_rate>[+-]\d+)fpm\s
    (?P<turn_rate>[+-][\d.]+)rot
""", re.VERBOSE | re.MULTILINE)

PATTERN_SKYLINES_POSITION_COMMENT = re.compile(r"""
    id(?P<skylines_id>\d+)\s
    (?P<climb_rate>[+-]\d+)fpm
""", re.VERBOSE | re.MULTILINE)

PATTERN_SPIDER_POSITION_COMMENT = re.compile(r"""
    id(?P<spider_id>[\d-]+)\s
    (?P<signal_power>[+-]\d+)dB\s
    (?P<spider_registration>[A-Z0-9]+)\s
    (?P<gps_quality>.+)
""", re.VERBOSE | re.MULTILINE)

PATTERN_SPOT_POSITION_COMMENT = re.compile(r"""
    id(?P<spot_id>[\d-]+)\s
    (?P<model>SPOT[A-Z\d]+)\s
    (?P<status>[A-Z]+)
""", re.VERBOSE | re.MULTILINE)

PATTERN_INREACH_POSITION_COMMENT = re.compile(r"""
    id(?P<id>[\d]+)\s
    (?P<model>inReac[A-Za-z\d]*)\s
    (?P<status>[A-Za-z]+)\s?
    (?P<pilot_name>.+)?
""", re.VERBOSE | re.MULTILINE)

PATTERN_TRACKER_POSITION_COMMENT = re.compile(r"""
    id(?P<details>[\dA-F]{2})(?P<address>[\dA-F]{6}?)\s?
    (?:(?P<climb_rate>[+-]\d+?)fpm\s)?
    (?:(?P<turn_rate>[+-][\d.]+?)rot\s)?
    (?:FL(?P<flight_level>[\d.]+)\s)?
    (?:(?P<signal_quality>[\d.]+?)dB\s)?
    (?:(?P<error_count>\d+)e\s)?
    (?:(?P<frequency_offset>[+-][\d.]+?)kHz\s?)?
    (?:gps(?P<gps_quality>(?P<gps_quality_horizontal>(\d+))x(?P<gps_quality_vertical>(\d+)))\s?)?
    (?:(?P<signal_power>[+-][\d.]+)dBm\s?)?
""", re.VERBOSE | re.MULTILINE)

PATTERN_TRACKER_STATUS_COMMENT = re.compile(r"""
    h(?P<hardware_version>[\d]{2})\s
    v(?P<software_version>[\d]{2})\s?
    (?:(?P<gps_satellites>[\d]+)sat/(?P<gps_quality>\d)\s?)?
    (?:(?P<gps_altitude>\d+)m\s?)?
    (?:(?P<pressure>[\d.]+)hPa\s?)?
    (?:(?P<temperature>[+-][\d.]+)degC\s?)?
    (?:(?P<humidity>\d+)%\s?)?
    (?:(?P<voltage>[\d.]+)V\s?)?
    (?:(?P<transmitter_power>\d+)/(?P<noise_level>[+-][\d.]+)dBm\s?)?
    (?:(?P<relays>\d+)/min)?
""", re.VERBOSE | re.MULTILINE)

PATTERN_RECEIVER_POSITION_COMMENT = re.compile(r"""
    (?:(?P<user_comment>.+))?
""", re.VERBOSE | re.MULTILINE)

PATTERN_RECEIVER_STATUS_COMMENT = re.compile(r"""
    (?:
        v(?P<version>\d+\.\d+\.\d+)
        (?:\.(?P<platform>.+?))?
    \s)?
    CPU:(?P<cpu_load>[\d.]+)\s
    RAM:(?P<ram_free>[\d.]+)/(?P<ram_total>[\d.]+)MB\s
    NTP:(?P<ntp_offset>[\d.]+)ms/(?P<ntp_correction>[+-][\d.]+)ppm\s
    (?:(?P<voltage>[\d.]+)V\s)?
    (?:(?P<amperage>[\d.]+)A\s)?
    (?:(?P<cpu_temperature>[+-][\d.]+)C\s*)?
    (?:(?P<visible_senders>\d+)/(?P<senders>\d+)Acfts\[1h\]\s*)?
    (?:RF:
        (?:
            (?P<rf_correction_manual>[+-][\d]+)
            (?P<rf_correction_automatic>[+-][\d.]+)ppm/
        )?
        (?P<signal_quality>[+-][\d.]+)dB
        (?:/(?P<senders_signal_quality>[+-][\d.]+)dB@10km\[(?P<senders_messages>\d+)\])?
        (?:/(?P<good_senders_signal_quality>[+-][\d.]+)dB@10km\[(?P<good_senders>\d+)/(?P<good_and_bad_senders>\d+)\])?
    )?
""", re.VERBOSE | re.MULTILINE)

PATTERN_TELNET_50001 = re.compile(r"""
    (?P<pps_offset>\d\.\d+)sec:(?P<frequency>\d+\.\d+)MHz:\s+
    (?P<aircraft_type>\d):(?P<address_type>\d):(?P<address>[A-F0-9]{6})\s
    (?P<timestamp>\d{6}):\s
    \[\s*(?P<latitude>[+-]\d+\.\d+),\s*(?P<longitude>[+-]\d+\.\d+)\]deg\s*
    (?P<altitude>\d+)m\s*
    (?P<climb_rate>[+-]\d+\.\d+)m/s\s*
    (?P<ground_speed>\d+\.\d+)m/s\s*
    (?P<track>\d+\.\d+)deg\s*
    (?P<turn_rate>[+-]\d+\.\d+)deg/sec\s*
    (?P<magic_number>\d+)\s*
    (?P<gps_status>[0-9x]+)m\s*
    (?P<channel>\d+)(?P<flarm_timeslot>[f_])(?P<ogn_timeslot>[o_])\s*
    (?P<frequency_offset>[+-]\d+\.\d+)kHz\s*
    (?P<decode_quality>\d+\.\d+)/(?P<signal_quality>\d+\.\d+)dB/(?P<demodulator_type>\d+)\s+
    (?P<error_count>\d+)e\s*
    (?P<distance>\d+\.\d+)km\s*
    (?P<bearing>\d+\.\d+)deg\s*
    (?P<phi>[+-]\d+\.\d+)deg\s*
    (?P<multichannel>\+)?\s*
    \?\s*
    R?\s*
    (B(?P<baro_altitude>\d+))?
""", re.VERBOSE | re.MULTILINE)

# The following regexp patterns are part of the ruby ogn-client.
# source: https://github.com/svoop/ogn_client-ruby

# The MIT License (MIT)
#
# Copyright (c) 2015-2017 Sven Schwyn
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

PATTERN_RECEIVER_BEACON = re.compile(r"""
    (?:
        v(?P<version>\d+\.\d+\.\d+)
        (?:\.(?P<platform>.+?))?
    \s)?
    CPU:(?P<cpu_load>[\d.]+)\s
    RAM:(?P<ram_free>[\d.]+)/(?P<ram_total>[\d.]+)MB\s
    NTP:(?P<ntp_offset>[\d.]+)ms/(?P<ntp_correction>[+-][\d.]+)ppm\s?
    (?:(?P<voltage>[\d.]+)V\s)?
    (?:(?P<amperage>[\d.]+)A\s)?
    (?:(?P<cpu_temperature>[+-][\d.]+)C\s*)?
    (?:(?P<visible_senders>\d+)/(?P<senders>\d+)Acfts\[1h\]\s*)?
    (Lat\:(?P<latency>\d+\.\d+)s\s*)?
    (?:RF:
        (?:
            (?P<rf_correction_manual>[+-][\d]+)
            (?P<rf_correction_automatic>[+-][\d.]+)ppm/
        )?
        (?P<signal_quality>[+-][\d.]+)dB
        (?:/(?P<senders_signal_quality>[+-][\d.]+)dB@10km\[(?P<senders_messages>\d+)\])?
        (?:/(?P<good_senders_signal_quality>[+-][\d.]+)dB@10km\[(?P<good_senders>\d+)/(?P<good_and_bad_senders>\d+)\])?
    )?
""", re.VERBOSE | re.MULTILINE)


PATTERN_AIRCRAFT_BEACON = re.compile(r"""
    id(?P<details>[\dA-F]{2})(?P<address>[\dA-F]{6}?)\s?
    (?:(?P<climb_rate>[+-]\d+?)fpm\s)?
    (?:(?P<turn_rate>[+-][\d.]+?)rot\s)?
    (?:FL(?P<flight_level>[\d.]+)\s)?
    (?:(?P<signal_quality>[\d.]+?)dB\s)?
    (?:(?P<errors>\d+)e\s)?
    (?:(?P<frequency_offset>[+-][\d.]+?)kHz\s?)?
    (?:gps(?P<gps_quality>(?P<gps_quality_horizontal>(\d+))x(?P<gps_quality_vertical>(\d+)))\s?)?
    (?:s(?P<flarm_software_version>[\d.]+)\s?)?
    (?:h(?P<flarm_hardware_version>[\dA-F]{2})\s?)?
    (?:r(?P<flarm_id>[\dA-F]+)\s?)?
    (?:(?P<signal_power>[+-][\d.]+)dBm\s?)?
    (?:(?P<proximity>(hear[\dA-F]{4}\s?)+))?
""", re.VERBOSE | re.MULTILINE)



import re
from datetime import datetime

from ogn.parser.utils import createTimestamp, parseAngle, KNOTS_TO_MS, KPH_TO_MS, FEETS_TO_METER, fahrenheit_to_celsius, CheapRuler, normalized_quality
from ogn.parser.pattern import PATTERN_APRS, PATTERN_APRS_POSITION, PATTERN_APRS_POSITION_WEATHER, PATTERN_APRS_STATUS, PATTERN_SERVER
from ogn.parser.exceptions import AprsParseError

from ogn.parser.aprs_comment.ogn_parser import OgnParser
from ogn.parser.aprs_comment.fanet_parser import FanetParser
from ogn.parser.aprs_comment.lt24_parser import LT24Parser
from ogn.parser.aprs_comment.naviter_parser import NaviterParser
from ogn.parser.aprs_comment.flarm_parser import FlarmParser
from ogn.parser.aprs_comment.tracker_parser import TrackerParser
from ogn.parser.aprs_comment.receiver_parser import ReceiverParser
from ogn.parser.aprs_comment.skylines_parser import SkylinesParser
from ogn.parser.aprs_comment.spider_parser import SpiderParser
from ogn.parser.aprs_comment.spot_parser import SpotParser
from ogn.parser.aprs_comment.inreach_parser import InreachParser
from ogn.parser.aprs_comment.generic_parser import GenericParser

positions = {}


def parse(aprs_message, reference_timestamp=None, calculate_relations=False):
    global positions

    if reference_timestamp is None:
        reference_timestamp = datetime.utcnow()

    message = parse_aprs(aprs_message, reference_timestamp)
    if message['aprs_type'] == 'position' or message['aprs_type'] == 'status':
        message.update(parse_comment(message['comment'],
                                     dstcall=message['dstcall'],
                                     aprs_type=message['aprs_type']))

    if message['aprs_type'].startswith('position') and calculate_relations is True:
        positions[message['name']] = (message['longitude'], message['latitude'])
        if message['receiver_name'] in positions:
            cheap_ruler = CheapRuler((message['latitude'] + positions[message['receiver_name']][1]) / 2.0)
            message['distance'] = cheap_ruler.distance((message['longitude'], message['latitude']), positions[message['receiver_name']])
            message['bearing'] = cheap_ruler.bearing((message['longitude'], message['latitude']), positions[message['receiver_name']])
            message['normalized_quality'] = normalized_quality(message['distance'], message['signal_quality']) if 'signal_quality' in message else None

    return message


def parse_aprs(message, reference_timestamp=None):
    if reference_timestamp is None:
        reference_timestamp = datetime.utcnow()

    result = {'raw_message': message,
              'reference_timestamp': reference_timestamp}

    if message and message[0] == '#':
        match_server = re.search(PATTERN_SERVER, message)
        if match_server:
            result.update({
                'version': match_server.group('version'),
                'timestamp': datetime.strptime(match_server.group('timestamp'), "%d %b %Y %H:%M:%S %Z"),
                'server': match_server.group('server'),
                'ip_address': match_server.group('ip_address'),
                'port': match_server.group('port'),
                'aprs_type': 'server'})
        else:
            result.update({
                'comment': message,
                'aprs_type': 'comment'})
    else:
        match = re.search(PATTERN_APRS, message)
        if match:
            aprs_type = 'position' if match.group('aprs_type') == '/' else 'status' if match.group('aprs_type') == '>' else 'unknown'
            result.update({'aprs_type': aprs_type})
            aprs_body = match.group('aprs_body')
            if aprs_type == 'position':
                match_position = re.search(PATTERN_APRS_POSITION, aprs_body)
                if match_position:
                    result.update({
                        'name': match.group('callsign'),
                        'dstcall': match.group('dstcall'),
                        'relay': match.group('relay') if match.group('relay') else None,
                        'receiver_name': match.group('receiver'),
                        'timestamp': createTimestamp(match_position.group('time'), reference_timestamp),
                        'latitude': parseAngle('0' + match_position.group('latitude') + (match_position.group('latitude_enhancement') or '0')) *   # noqa: W504
                        (-1 if match_position.group('latitude_sign') == 'S' else 1),
                        'symboltable': match_position.group('symbol_table'),
                        'longitude': parseAngle(match_position.group('longitude') + (match_position.group('longitude_enhancement') or '0')) *   # noqa: W504
                        (-1 if match_position.group('longitude_sign') == 'W' else 1),
                        'symbolcode': match_position.group('symbol'),

                        'track': int(match_position.group('course')) if match_position.group('course_extension') else None,
                        'ground_speed': int(match_position.group('ground_speed')) * KNOTS_TO_MS / KPH_TO_MS if match_position.group('ground_speed') else None,
                        'altitude': int(match_position.group('altitude')) * FEETS_TO_METER if match_position.group('altitude') else None,

                        'comment': match_position.group('comment') if match_position.group('comment') else "",
                    })
                    return result

                match_position_weather = re.search(PATTERN_APRS_POSITION_WEATHER, aprs_body)
                if match_position_weather:
                    result.update({
                        'aprs_type': 'position_weather',

                        'name': match.group('callsign'),
                        'dstcall': match.group('dstcall'),
                        'relay': match.group('relay') if match.group('relay') else None,
                        'receiver_name': match.group('receiver'),
                        'timestamp': createTimestamp(match_position_weather.group('time'), reference_timestamp),
                        'latitude': parseAngle('0' + match_position_weather.group('latitude')) *   # noqa: W504
                        (-1 if match_position_weather.group('latitude_sign') == 'S' else 1),
                        'symboltable': match_position_weather.group('symbol_table'),
                        'longitude': parseAngle(match_position_weather.group('longitude')) *   # noqa: W504
                        (-1 if match_position_weather.group('longitude_sign') == 'W' else 1),
                        'symbolcode': match_position_weather.group('symbol'),

                        'wind_direction': int(match_position_weather.group('wind_direction')) if match_position_weather.group('wind_direction') != '...' else None,
                        'wind_speed': int(match_position_weather.group('wind_speed')) * KNOTS_TO_MS / KPH_TO_MS if match_position_weather.group('wind_speed') != '...' else None,
                        'wind_speed_peak': int(match_position_weather.group('wind_speed_peak')) * KNOTS_TO_MS / KPH_TO_MS if match_position_weather.group('wind_speed_peak') != '...' else None,
                        'temperature': fahrenheit_to_celsius(float(match_position_weather.group('temperature'))) if match_position_weather.group('temperature') != '...' else None,
                        'humidity': int(match_position_weather.group('humidity')) * 0.01 if match_position_weather.group('humidity') else None,
                        'barometric_pressure': int(match_position_weather.group('barometric_pressure')) if match_position_weather.group('barometric_pressure') else None,

                        'comment': match_position_weather.group('comment') if match_position_weather.group('comment') else "",
                    })
                    return result

                raise AprsParseError(message)
            elif aprs_type == 'status':
                match_status = re.search(PATTERN_APRS_STATUS, aprs_body)
                if match_status:
                    result.update({
                        'name': match.group('callsign'),
                        'dstcall': match.group('dstcall'),
                        'receiver_name': match.group('receiver'),
                        'timestamp': createTimestamp(match_status.group('time'), reference_timestamp),
                        'comment': match_status.group('comment') if match_status.group('comment') else ""})
                else:
                    raise NotImplementedError(message)
        else:
            raise AprsParseError(message)

    return result


dstcall_parser_mapping = {'APRS': OgnParser(),
                          'OGNFNT': FanetParser(),
                          'OGFLR': FlarmParser(),
                          'OGNTRK': TrackerParser(),
                          'OGNSDR': ReceiverParser(),
                          'OGCAPT': GenericParser(beacon_type='capturs'),
                          'OGFLYM': GenericParser(beacon_type='flymaster'),
                          'OGINRE': InreachParser(),
                          'OGLT24': LT24Parser(),
                          'OGNAVI': NaviterParser(),
                          'OGPAW': GenericParser(beacon_type='pilot_aware'),
                          'OGSKYL': SkylinesParser(),
                          'OGSPID': SpiderParser(),
                          'OGSPOT': SpotParser(),
                          'GENERIC': GenericParser(beacon_type='unknown'),
                          }


def parse_comment(aprs_comment, dstcall='APRS', aprs_type="position"):
    parser = dstcall_parser_mapping.get(dstcall)
    if parser:
        return parser.parse(aprs_comment, aprs_type)
    else:
        return dstcall_parser_mapping.get('GENERIC').parse(aprs_comment, aprs_type)



from ogn.parser.utils import FPM_TO_MS
from ogn.parser.pattern import PATTERN_FANET_POSITION_COMMENT, PATTERN_FANET_STATUS_COMMENT

from .base import BaseParser


class FanetParser(BaseParser):
    def __init__(self):
        self.beacon_type = 'fanet'
        self.position_pattern = PATTERN_FANET_POSITION_COMMENT
        self.status_pattern = PATTERN_FANET_STATUS_COMMENT

    def parse_position(self, aprs_comment):
        match = self.position_pattern.match(aprs_comment)
        result = {}
        if match.group('details'):
            result.update({
                'address_type': int(match.group('details'), 16) & 0b00000011,
                'aircraft_type': (int(match.group('details'), 16) & 0b01111100) >> 2,
                'stealth': (int(match.group('details'), 16) & 0b10000000) >> 7 == 1,
                'address': match.group('address')
            })
        if match.group('climb_rate'): result['climb_rate'] = int(match.group('climb_rate')) * FPM_TO_MS
        return result

    def parse_status(self, aprs_comment):
        match = self.status_pattern.match(aprs_comment)
        result = {}
        if match.group('fanet_name'): result['fanet_name'] = match.group('fanet_name')
        if match.group('signal_quality'): result['signal_quality'] = float(match.group('signal_quality'))
        if match.group('frequency_offset'): result['frequency_offset'] = float(match.group('frequency_offset'))
        if match.group('error_count'): result['error_count'] = int(match.group('error_count'))

        return result



"""
exception definitions
"""


class ParseError(Exception):
    pass


class AprsParseError(ParseError):
    """Parse error while parsing an aprs packet."""
    def __init__(self, aprs_string):
        self.aprs_string = aprs_string

        self.message = "This is not a valid APRS packet: {}".format(aprs_string)
        super(AprsParseError, self).__init__(self.message)


class OgnParseError(ParseError):
    """Parse error while parsing an ogn message from aprs comment."""
    def __init__(self, aprs_comment):
        self.aprs_comment = aprs_comment

        self.message = "This is not a valid OGN message: {}".format(aprs_comment)
        super(OgnParseError, self).__init__(self.message)



import unittest
import unittest.mock as mock

from ogn.client.client import TelnetClient


class TelnetClientTest(unittest.TestCase):
    @mock.patch('ogn.client.client.socket')
    def test_connect_disconnect(self, socket_mock):
        client = TelnetClient()
        client.connect()
        client.sock.connect.assert_called_once_with(('localhost', 50001))

        client.disconnect()
        client.sock.shutdown.assert_called_once_with(0)
        client.sock.close.assert_called_once_with()

    @mock.patch('ogn.client.client.socket')
    def test_run(self, socket_mock):
        def callback(raw_message):
            raise ConnectionRefusedError

        client = TelnetClient()
        client.connect()

        client.run(callback=callback)



import unittest

from ogn.parser.aprs_comment.spider_parser import SpiderParser


class TestStringMethods(unittest.TestCase):
    def test_position_comment(self):
        message = SpiderParser().parse_position("id300234060668560 +30dB K23W 3D")

        self.assertEqual(message['spider_id'], "300234060668560")
        self.assertEqual(message['signal_power'], 30)
        self.assertEqual(message['spider_registration'], "K23W")
        self.assertEqual(message['gps_quality'], "3D")


if __name__ == '__main__':
    unittest.main()



import unittest

from ogn.parser.aprs_comment.generic_parser import GenericParser


class TestStringMethods(unittest.TestCase):
    def test_position_comment(self):
        message = GenericParser().parse_position("id0123456789 weather is good, climbing with 123fpm")
        self.assertTrue('comment' in message)

        message = GenericParser().parse_status("id0123456789 weather is good, climbing with 123fpm")
        self.assertTrue('comment' in message)


if __name__ == '__main__':
    unittest.main()



import unittest
from datetime import datetime, timezone

from ogn.parser.utils import parseAngle, createTimestamp, CheapRuler, normalized_quality


class TestStringMethods(unittest.TestCase):
    def test_parseAngle(self):
        self.assertAlmostEqual(parseAngle('05048.30'), 50.805, 5)

    def proceed_test_data(self, test_data={}):
        for test in test_data:
            timestamp = createTimestamp(test[0], reference_timestamp=test[1])
            self.assertEqual(timestamp, test[2])

    def test_createTimestamp_hhmmss(self):
        test_data = [
            ('000001h', datetime(2015, 1, 10, 0, 0, 1), datetime(2015, 1, 10, 0, 0, 1)),        # packet from current day (on the tick)
            ('235959h', datetime(2015, 1, 10, 0, 0, 1), datetime(2015, 1, 9, 23, 59, 59)),      # packet from previous day (2 seconds old)
            ('110000h', datetime(2015, 1, 10, 0, 0, 1), datetime(2015, 1, 10, 11, 0, 0)),       # packet 11 hours from future or 13 hours old
            ('123500h', datetime(2015, 1, 10, 23, 50, 0), datetime(2015, 1, 10, 12, 35, 0)),    # packet from current day (11 hours old)
            ('000001h', datetime(2015, 1, 10, 23, 50, 0), datetime(2015, 1, 11, 0, 0, 1)),      # packet from next day (11 minutes from future)
        ]

        self.proceed_test_data(test_data)

    def test_createTimestamp_ddhhmm(self):
        test_data = [
            ('011212z', datetime(2017, 9, 28, 0, 0, 1), datetime(2017, 10, 1, 12, 12, 0)),      # packet from 1st of month, received on september 28th,
            ('281313z', datetime(2017, 10, 1, 0, 0, 1), datetime(2017, 9, 28, 13, 13, 0)),      # packet from 28th of month, received on october 1st,
            ('281414z', datetime(2017, 1, 1, 0, 0, 1), datetime(2016, 12, 28, 14, 14, 0)),      # packet from 28th of month, received on january 1st,
        ]

        self.proceed_test_data(test_data)

    def test_createTimestamp_tzinfo(self):
        test_data = [
            ('000001h', datetime(2020, 9, 10, 0, 0, 1, tzinfo=timezone.utc), (datetime(2020, 9, 10, 0, 0, 1, tzinfo=timezone.utc)))
        ]

        self.proceed_test_data(test_data)

    def test_cheap_ruler_distance(self):
        koenigsdf = (11.465353, 47.829825)
        hochkoenig = (13.062405, 47.420516)

        cheap_ruler = CheapRuler((koenigsdf[1] + hochkoenig[1]) / 2)
        distance = cheap_ruler.distance(koenigsdf, hochkoenig)
        self.assertAlmostEqual(distance, 128381.47612138899)

    def test_cheap_ruler_bearing(self):
        koenigsdf = (11.465353, 47.829825)
        hochkoenig = (13.062405, 47.420516)

        cheap_ruler = CheapRuler((koenigsdf[1] + hochkoenig[1]) / 2)
        bearing = cheap_ruler.bearing(koenigsdf, hochkoenig)
        self.assertAlmostEqual(bearing, 110.761300063515)

    def test_normalized_quality(self):
        self.assertAlmostEqual(normalized_quality(10000, 1), 1)
        self.assertAlmostEqual(normalized_quality(20000, 10), 16.020599913279625)
        self.assertAlmostEqual(normalized_quality(5000, 5), -1.0205999132796242)


if __name__ == '__main__':
    unittest.main()



from datetime import datetime, timedelta, timezone
import math

FEETS_TO_METER = 0.3048             # ratio feets to meter
FPM_TO_MS = FEETS_TO_METER / 60     # ratio fpm to m/s
KNOTS_TO_MS = 0.5144                # ratio knots to m/s
KPH_TO_MS = 0.27778                 # ratio kph to m/s
HPM_TO_DEGS = 180 / 60              # ratio between half turn per minute and degrees/s


def fahrenheit_to_celsius(fahrenheit):
    return (fahrenheit - 32.0) * 5.0 / 9.0


def parseAngle(dddmmhht):
    return float(dddmmhht[:3]) + float(dddmmhht[3:]) / 60


def createTimestamp(time_string, reference_timestamp):
    if time_string[-1] == "z":
        dd = int(time_string[0:2])
        hh = int(time_string[2:4])
        mm = int(time_string[4:6])

        result = datetime(reference_timestamp.year,
                          reference_timestamp.month,
                          dd,
                          hh, mm, 0,
                          tzinfo=timezone.utc if reference_timestamp.tzinfo is not None else None)

        # correct wrong month
        if result > reference_timestamp + timedelta(days=14):
            result = (result.replace(day=1) - timedelta(days=14)).replace(day=result.day)
        elif result < reference_timestamp - timedelta(days=14):
            result = (result.replace(day=28) + timedelta(days=14)).replace(day=result.day)
    else:
        hh = int(time_string[0:2])
        mm = int(time_string[2:4])
        ss = int(time_string[4:6])

        result = datetime(reference_timestamp.year,
                          reference_timestamp.month,
                          reference_timestamp.day,
                          hh, mm, ss,
                          tzinfo=timezone.utc if reference_timestamp.tzinfo is not None else None)

        if result > reference_timestamp + timedelta(hours=12):
            # shift timestamp to previous day
            result -= timedelta(days=1)
        elif result < reference_timestamp - timedelta(hours=12):
            # shift timestamp to next day
            result += timedelta(days=1)

    return result


MATH_PI = 3.14159265359


class CheapRuler():
    """Extreme fast distance calculating for distances below 500km."""

    def __init__(self, lat):
        c = math.cos(lat * MATH_PI / 180)
        c2 = 2 * c * c - 1
        c3 = 2 * c * c2 - c
        c4 = 2 * c * c3 - c2
        c5 = 2 * c * c4 - c3

        self.kx = 1000 * (111.41513 * c - 0.09455 * c3 + 0.00012 * c5)  # longitude correction
        self.ky = 1000 * (111.13209 - 0.56605 * c2 + 0.0012 * c4)       # latitude correction

    def distance(self, a, b):
        """Distance between point a and b. A point is a tuple(lon,lat)."""

        dx = (a[0] - b[0]) * self.kx
        dy = (a[1] - b[1]) * self.ky
        return math.sqrt(dx * dx + dy * dy)

    def bearing(self, a, b):
        """Returns the bearing from point a to point b."""

        dx = (b[0] - a[0]) * self.kx
        dy = (b[1] - a[1]) * self.ky
        if dx == 0 and dy == 0:
            return 0
        result = math.atan2(-dy, dx) * 180 / MATH_PI + 90
        return result if result >= 0 else result + 360


def normalized_quality(distance, signal_quality):
    """Signal quality normalized to 10km."""

    return signal_quality + 20.0 * math.log10(distance / 10000.0) if distance > 0 else None



import unittest

from ogn.parser.utils import FPM_TO_MS
from ogn.parser.aprs_comment.fanet_parser import FanetParser


class TestStringMethods(unittest.TestCase):
    def test_position_comment(self):
        message = FanetParser().parse_position("id1E1103CE -02fpm")

        self.assertEqual(message['address_type'], 2)
        self.assertEqual(message['aircraft_type'], 7)
        self.assertFalse(message['stealth'])
        self.assertEqual(message['address'], "1103CE")
        self.assertAlmostEqual(message['climb_rate'], -2 * FPM_TO_MS, 0.1)

    def test_pseudo_status_comment(self):
        message = FanetParser().parse_position("")

        self.assertEqual(message, {})

    def test_v028_status(self):
        message = FanetParser().parse_status('Name="Juerg Zweifel" 15.0dB -17.1kHz 1e')

        self.assertEqual(message['fanet_name'], "Juerg Zweifel")
        self.assertEqual(message['signal_quality'], 15.0)
        self.assertEqual(message['frequency_offset'], -17.1)
        self.assertEqual(message['error_count'], 1)


if __name__ == '__main__':
    unittest.main()



import unittest
import unittest.mock as mock

from ogn.parser import parse
from ogn.client.client import create_aprs_login, AprsClient
from ogn.client.settings import APRS_APP_NAME, APRS_APP_VER, APRS_KEEPALIVE_TIME


class AprsClientTest(unittest.TestCase):
    def test_create_aprs_login(self):
        basic_login = create_aprs_login('klaus', -1, 'myApp', '0.1')
        self.assertEqual('user klaus pass -1 vers myApp 0.1\n', basic_login)

        login_with_filter = create_aprs_login('klaus', -1, 'myApp', '0.1', 'r/48.0/11.0/100')
        self.assertEqual('user klaus pass -1 vers myApp 0.1 filter r/48.0/11.0/100\n', login_with_filter)

    def test_initialisation(self):
        client = AprsClient(aprs_user='testuser', aprs_filter='')
        self.assertEqual(client.aprs_user, 'testuser')
        self.assertEqual(client.aprs_filter, '')

    @mock.patch('ogn.client.client.socket')
    def test_connect_full_feed(self, mock_socket):
        client = AprsClient(aprs_user='testuser', aprs_filter='')
        client.connect()
        client.sock.send.assert_called_once_with('user testuser pass -1 vers {} {}\n'.format(
                                                 APRS_APP_NAME, APRS_APP_VER).encode('ascii'))
        client.sock.makefile.assert_called_once_with('rw')

    @mock.patch('ogn.client.client.socket')
    def test_connect_client_defined_filter(self, mock_socket):
        client = AprsClient(aprs_user='testuser', aprs_filter='r/50.4976/9.9495/100')
        client.connect()
        client.sock.send.assert_called_once_with('user testuser pass -1 vers {} {} filter r/50.4976/9.9495/100\n'.format(
                                                 APRS_APP_NAME, APRS_APP_VER).encode('ascii'))
        client.sock.makefile.assert_called_once_with('rw')

    @mock.patch('ogn.client.client.socket')
    def test_disconnect(self, mock_socket):
        client = AprsClient(aprs_user='testuser', aprs_filter='')
        client.connect()
        client.disconnect()
        client.sock.shutdown.assert_called_once_with(0)
        client.sock.close.assert_called_once_with()
        self.assertTrue(client._kill)

    @mock.patch('ogn.client.client.socket')
    def test_run(self, mock_socket):
        import socket
        mock_socket.error = socket.error

        client = AprsClient(aprs_user='testuser', aprs_filter='')
        client.connect()

        client.sock_file.readline = mock.MagicMock()
        client.sock_file.readline.side_effect = ['Normal text blabla',
                                                 'my weird character ¥',
                                                 UnicodeDecodeError('funnycodec', b'\x00\x00', 1, 2, 'This is just a fake reason!'),
                                                 '... show must go on',
                                                 BrokenPipeError(),
                                                 '... and on',
                                                 ConnectionResetError(),
                                                 '... and on',
                                                 socket.error(),
                                                 '... and on',
                                                 '',
                                                 '... and on',
                                                 KeyboardInterrupt()]

        try:
            client.run(callback=lambda msg: print("got: {}".format(msg)), autoreconnect=True)
        except KeyboardInterrupt:
            pass
        finally:
            client.disconnect()

    @mock.patch('ogn.client.client.time')
    @mock.patch('ogn.client.client.socket')
    def test_run_keepalive(self, mock_socket, mock_time):
        import socket
        mock_socket.error = socket.error

        client = AprsClient(aprs_user='testuser', aprs_filter='')
        client.connect()

        client.sock_file.readline = mock.MagicMock()
        client.sock_file.readline.side_effect = ['Normal text blabla',
                                                 KeyboardInterrupt()]

        mock_time.side_effect = [0, 0, APRS_KEEPALIVE_TIME + 1, APRS_KEEPALIVE_TIME + 1]

        timed_callback = mock.MagicMock()

        try:
            client.run(callback=lambda msg: print("got: {}".format(msg)), timed_callback=timed_callback)
        except KeyboardInterrupt:
            pass
        finally:
            client.disconnect()

        timed_callback.assert_called_with(client)

    def test_reset_kill_reconnect(self):
        client = AprsClient(aprs_user='testuser', aprs_filter='')
        client.connect()

        # .run() should be allowed to execute after .connect()
        mock_callback = mock.MagicMock(
            side_effect=lambda raw_msg: client.disconnect())

        self.assertFalse(client._kill)
        client.run(callback=mock_callback, autoreconnect=True)

        # After .disconnect(), client._kill should be True
        self.assertTrue(client._kill)
        self.assertEqual(mock_callback.call_count, 1)

        # After we reconnect, .run() should be able to run again
        mock_callback.reset_mock()
        client.connect()
        client.run(callback=mock_callback, autoreconnect=True)
        self.assertEqual(mock_callback.call_count, 1)

    def test_50_live_messages(self):
        print("Enter")
        self.remaining_messages = 50

        def process_message(raw_message):
            if raw_message[0] == '#':
                return
            try:
                message = parse(raw_message)
                print("{}: {}".format(message['aprs_type'], raw_message))
            except NotImplementedError as e:
                print("{}: {}".format(e, raw_message))
                return
            if self.remaining_messages > 0:
                self.remaining_messages -= 1
            else:
                raise KeyboardInterrupt

        client = AprsClient(aprs_user='testuser', aprs_filter='')
        client.connect()
        try:
            client.run(callback=process_message, autoreconnect=True)
        except KeyboardInterrupt:
            pass
        finally:
            client.disconnect()
        self.assertTrue(True)



import unittest
import os

from datetime import datetime

from ogn.parser.parse import parse


class TestStringMethods(unittest.TestCase):
    def test_ogn_client(self):
        with open(os.path.dirname(__file__) + '/valid_beacon_data/oneminute.txt') as f:
            for line in f:
                timestamp = datetime.strptime(line[:26], '%Y-%m-%d %H:%M:%S.%f')
                aprs_string = line[28:]
                message = parse(aprs_string, reference_timestamp=timestamp, calculate_relations=True)
                self.assertFalse(message is None)

    def test_ogn_lib(self):
        from ogn_lib import Parser
        
        with open(os.path.dirname(__file__) + '/valid_beacon_data/oneminute.txt') as f:
            for line in f:
                aprs_string = line[28:]
                if aprs_string.startswith('#'):
                    continue
                
                try:
                    message = Parser.parse_message(aprs_string)
                    self.assertFalse(message is None)
                except Exception:
                    print(aprs_string)



if __name__ == '__main__':
    unittest.main()



class BaseParser():
    def __init__(self):
        self.beacon_type = 'unknown'

    def parse(self, aprs_comment, aprs_type):
        if aprs_type.startswith('position'):
            data = self.parse_position(aprs_comment)
        elif aprs_type.startswith('status'):
            data = self.parse_status(aprs_comment)
        else:
            raise ValueError("aprs_type {} unknown".format(aprs_type))
        data.update({'beacon_type': self.beacon_type})
        return data

    def parse_position(self, aprs_comment):
        raise NotImplementedError("Position parser for parser '{}' not yet implemented".format(self.beacon_type))

    def parse_status(self, aprs_comment):
        raise NotImplementedError("Status parser for parser '{}' not yet implemented".format(self.beacon_type))



from .base import BaseParser


class GenericParser(BaseParser):
    def __init__(self, beacon_type='unknown'):
        self.beacon_type = beacon_type

    def parse_position(self, aprs_comment):
        return {'comment': aprs_comment}

    def parse_status(self, aprs_comment):
        return {'comment': aprs_comment}



import unittest

from ogn.parser.utils import FPM_TO_MS, HPM_TO_DEGS
from ogn.parser.aprs_comment.naviter_parser import NaviterParser


class TestStringMethods(unittest.TestCase):
    def test_OGNAVI_1(self):
        message = NaviterParser().parse_position("id0440042121 +123fpm +0.5rot")

        # id0440042121 == 0b0000 0100 0100 0000 0000 0100 0010 0001 0010 0001
        # bit 0: stealth mode
        # bit 1: do not track mode
        # bits 2-5: aircraft type
        # bits 6-11: address type (namespace is extended from 2 to 6 bits to avoid collisions with other tracking providers)
        # bits 12-15: reserved for use at a later time
        # bits 16-39: device id (24-bit device identifier, same as in APRS header)
        self.assertEqual(message['stealth'], False)
        self.assertEqual(message['do_not_track'], False)
        self.assertEqual(message['aircraft_type'], 1)
        self.assertEqual(message['address_type'], 4)
        self.assertEqual(message['reserved'], 0)
        self.assertEqual(message['address'], "042121")

        self.assertAlmostEqual(message['climb_rate'], 123 * FPM_TO_MS, 2)
        self.assertEqual(message['turn_rate'], 0.5 * HPM_TO_DEGS)


if __name__ == '__main__':
    unittest.main()



from ogn.parser.pattern import PATTERN_RECEIVER_POSITION_COMMENT, PATTERN_RECEIVER_STATUS_COMMENT

from .base import BaseParser


class ReceiverParser(BaseParser):
    def __init__(self):
        self.beacon_type = 'receiver'
        self.position_pattern = PATTERN_RECEIVER_POSITION_COMMENT
        self.status_pattern = PATTERN_RECEIVER_STATUS_COMMENT

    def parse_position(self, aprs_comment):
        if aprs_comment is None:
            return {}
        else:
            match = self.position_pattern.match(aprs_comment)
            return {'user_comment': match.group('user_comment') if match.group('user_comment') else None}

    def parse_status(self, aprs_comment):
        match = self.status_pattern.match(aprs_comment)
        result = {
            'version': match.group('version'),
            'platform': match.group('platform'),
            'cpu_load': float(match.group('cpu_load')),
            'free_ram': float(match.group('ram_free')),
            'total_ram': float(match.group('ram_total')),
            'ntp_error': float(match.group('ntp_offset')),
        }

        if match.group('ntp_correction'): result['rt_crystal_correction'] = float(match.group('ntp_correction'))
        if match.group('voltage'): result['voltage'] = float(match.group('voltage'))
        if match.group('amperage'): result['amperage'] = float(match.group('amperage'))
        if match.group('cpu_temperature'): result['cpu_temp'] = float(match.group('cpu_temperature'))
        if match.group('visible_senders'): result['senders_visible'] = int(match.group('visible_senders'))
        if match.group('senders'): result['senders_total'] = int(match.group('senders'))
        if match.group('rf_correction_manual'): result['rec_crystal_correction'] = int(match.group('rf_correction_manual'))
        if match.group('rf_correction_automatic'): result['rec_crystal_correction_fine'] = float(match.group('rf_correction_automatic'))
        if match.group('signal_quality'): result['rec_input_noise'] = float(match.group('signal_quality'))
        if match.group('senders_signal_quality'): result['senders_signal'] = float(match.group('senders_signal_quality'))
        if match.group('senders_messages'): result['senders_messages'] = float(match.group('senders_messages'))
        if match.group('good_senders_signal_quality'): result['good_senders_signal'] = float(match.group('good_senders_signal_quality'))
        if match.group('good_senders'): result['good_senders'] = float(match.group('good_senders'))
        if match.group('good_and_bad_senders'): result['good_and_bad_senders'] = float(match.group('good_and_bad_senders'))

        return result



from ogn.parser.pattern import PATTERN_SPIDER_POSITION_COMMENT

from .base import BaseParser


class SpiderParser(BaseParser):
    def __init__(self):
        self.beacon_type = 'spider'
        self.position_pattern = PATTERN_SPIDER_POSITION_COMMENT

    def parse_position(self, aprs_comment):
        match = self.position_pattern.match(aprs_comment)
        return {'spider_id': match.group('spider_id'),
                'signal_power': int(match.group('signal_power')) if match.group('signal_power') else None,
                'spider_registration': match.group('spider_registration') if match.group('spider_registration') else None,
                'gps_quality': match.group('gps_quality') if match.group('gps_quality') else None}



import unittest

from ogn.parser.utils import FPM_TO_MS, HPM_TO_DEGS
from ogn.parser.aprs_comment.tracker_parser import TrackerParser


class TestStringMethods(unittest.TestCase):
    def test_position_comment(self):
        message = TrackerParser().parse_position("id072FD00F -058fpm +1.1rot FL003.12 32.8dB 0e -0.8kHz gps3x5 +12.7dBm")

        self.assertEqual(message['address_type'], 3)
        self.assertEqual(message['aircraft_type'], 1)
        self.assertFalse(message['stealth'])
        self.assertEqual(message['address'], "2FD00F")
        self.assertAlmostEqual(message['climb_rate'], -58 * FPM_TO_MS, 2)
        self.assertEqual(message['turn_rate'], 1.1 * HPM_TO_DEGS)
        self.assertEqual(message['flightlevel'], 3.12)
        self.assertEqual(message['signal_quality'], 32.8)
        self.assertEqual(message['error_count'], 0)
        self.assertEqual(message['frequency_offset'], -0.8)
        self.assertEqual(message['gps_quality'], {'horizontal': 3, 'vertical': 5})
        self.assertEqual(message['signal_power'], 12.7)

    def test_status_comment(self):
        message = TrackerParser().parse_status("h00 v00 9sat/1 164m 1002.6hPa +20.2degC 0% 3.34V 14/-110.5dBm 1/min")

        self.assertEqual(message['hardware_version'], 0)
        self.assertEqual(message['software_version'], 0)
        self.assertEqual(message['gps_satellites'], 9)
        self.assertEqual(message['gps_quality'], 1)
        self.assertEqual(message['gps_altitude'], 164)
        self.assertEqual(message['pressure'], 1002.6)
        self.assertEqual(message['temperature'], 20.2)
        self.assertEqual(message['humidity'], 0)
        self.assertEqual(message['voltage'], 3.34)
        self.assertEqual(message['transmitter_power'], 14)
        self.assertEqual(message['noise_level'], -110.5)
        self.assertEqual(message['relays'], 1)

    def test_status_comment_comment(self):
        message = TrackerParser().parse_status("Pilot=Pawel Hard=DIY/STM32")

        self.assertEqual(message['comment'], "Pilot=Pawel Hard=DIY/STM32")


if __name__ == '__main__':
    unittest.main()



import unittest

from ogn.parser.aprs_comment.spot_parser import SpotParser


class TestStringMethods(unittest.TestCase):
    def test_position_comment(self):
        message = SpotParser().parse_position("id0-2860357 SPOT3 GOOD")

        self.assertEqual(message['spot_id'], "0-2860357")
        self.assertEqual(message['model'], 'SPOT3')
        self.assertEqual(message['status'], "GOOD")


if __name__ == '__main__':
    unittest.main()
